@@ -1,6 +1,6 @@
 
 /*
- * $Id: neighbors.c,v 1.255.2.3 1998/10/07 20:01:20 wessels Exp $
+ * $Id: neighbors.c,v 1.258.2.2 1998/11/13 07:24:08 wessels Exp $
  *
  * DEBUG: section 15    Neighbor Routines
  * AUTHOR: Harvest Derived
@@ -274,6 +274,22 @@ getDefaultParent(request_t * request)
     return NULL;
 }
 
+peer *
+getAnyParent(request_t * request)
+{
+    peer *p = NULL;
+    for (p = Config.peers; p; p = p->next) {
+	if (neighborType(p, request) != PEER_PARENT)
+	    continue;
+	if (!peerHTTPOkay(p, request))
+	    continue;
+	debug(15, 3) ("getAnyParent: returning %s\n", p->host);
+	return p;
+    }
+    debug(15, 3) ("getAnyParent: returning NULL\n");
+    return NULL;
+}
+
 peer *
 getNextPeer(peer * p)
 {
@@ -746,7 +762,7 @@ neighborsUdpAck(const cache_key * key, icp_common_t * header, const struct socka
 	return;
     }
     if (entry->ping_status != PING_WAITING) {
-	debug(15, 2) ("neighborsUdpAck: Unexpected %s for %s\n",
+	debug(15, 2) ("neighborsUdpAck: Late %s for %s\n",
 	    opcode_d, storeKeyText(key));
 	neighborCountIgnored(p);
 	return;
@@ -1100,6 +1116,8 @@ dump_peer_options(StoreEntry * sentry, peer * p)
     if (p->options.no_delay)
 	storeAppendPrintf(sentry, " no-delay");
 #endif
+    if (p->login)
+	storeAppendPrintf(sentry, " login=%s", p->login);
     if (p->mcast.ttl > 0)
 	storeAppendPrintf(sentry, " ttl=%d", p->mcast.ttl);
     storeAppendPrintf(sentry, "\n");
