@@ -1,8 +1,9 @@
 
 
 
+
 /*
- * $Id: structs.h,v 1.236.2.1 1998/10/07 20:01:18 wessels Exp $
+ * $Id: structs.h,v 1.245.2.11 1998/11/11 23:16:27 wessels Exp $
  *
  *
  * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
@@ -58,11 +59,6 @@ struct _acl_name_list {
     acl_name_list *next;
 };
 
-struct _acl_proxy_auth {
-    int timeout;		/* timeout value for cached usercode:password entries */
-    hash_table *hash;
-};
-
 struct _acl_proxy_auth_user {
     /* first two items must be same as hash_link */
     char *user;
@@ -296,6 +292,7 @@ struct _SquidConfig {
     int dnsChildren;
     int redirectChildren;
     int authenticateChildren;
+    int authenticateTTL;
     struct {
 	char *host;
 	u_short port;
@@ -377,6 +374,9 @@ struct _SquidConfig {
 #if HTTP_VIOLATIONS
 	int reload_into_ims;
 #endif
+	int offline;
+	int redir_rewrites_host;
+	int persistent_client_posts;
     } onoff;
     acl *aclList;
     struct {
@@ -439,6 +439,8 @@ struct _SquidConfig {
 	int http_min_poll;
     } comm_incoming;
     int max_open_disk_fds;
+    int uri_whitespace;
+    size_t rangeOffsetLimit;
 };
 
 struct _SquidConfig2 {
@@ -464,16 +466,8 @@ struct _dread_ctrl {
     void *client_data;
 };
 
-struct _helper_flags {
-    unsigned int alive:1;
-    unsigned int busy:1;
-    unsigned int closing:1;
-    unsigned int shutdown:1;
-};
-
 struct _dnsserver_t {
     int id;
-    helper_flags flags;
     int inpipe;
     int outpipe;
     time_t answer;
@@ -769,6 +763,7 @@ struct _AccessLogEntry {
 	method_t method;
 	int code;
 	const char *content_type;
+	float version;
     } http;
     struct {
 	icp_opcode opcode;
@@ -820,6 +815,10 @@ struct _clientHttpRequest {
 	unsigned int internal:1;
 	unsigned int done_copying:1;
     } flags;
+    struct {
+	http_status status;
+	char *location;
+    } redirect;
 };
 
 struct _ConnStateData {
@@ -905,6 +904,7 @@ struct _fqdncache_entry {
     char *names[FQDN_MAX_NAMES + 1];
     fqdn_pending *pending_head;
     char *error_message;
+    struct timeval request_time;
     dlink_node lru;
     unsigned char locks;
     fqdncache_status_t status:3;
@@ -922,6 +922,7 @@ struct _domain_type {
     domain_type *next;
 };
 
+#if USE_CACHE_DIGESTS
 struct _Version {
     short int current;		/* current version */
     short int required;		/* minimal version that can safely handle current version */
@@ -982,6 +983,8 @@ struct _PeerDigest {
     } stats;
 };
 
+#endif
+
 struct _peer {
     char *host;
     peer_t type;
@@ -1042,7 +1045,9 @@ struct _peer {
 	    unsigned int counting:1;
 	} flags;
     } mcast;
+#if USE_CACHE_DIGESTS
     PeerDigest digest;
+#endif
     int tcp_up;			/* 0 if a connect() fails */
     time_t last_fail_time;
     struct in_addr addresses[10];
@@ -1057,6 +1062,7 @@ struct _peer {
 	float load_factor;
     } carp;
 #endif
+    char *login;		/* Proxy authorization */
 };
 
 struct _net_db_name {
@@ -1108,7 +1114,6 @@ struct _ps_state {
      */
     struct sockaddr_in first_parent_miss;
     struct sockaddr_in closest_parent_miss;
-    struct sockaddr_in single_parent;
     ping_data ping;
     aclCheck_t *acl_checklist;
 };
@@ -1269,6 +1274,8 @@ struct _request_flags {
 #if HTTP_VIOLATIONS
     unsigned int nocache_hack:1;	/* for changing/ignoring no-cache requests */
 #endif
+    unsigned int accelerated:1;
+    unsigned int internal:1;
 };
 
 struct _request_t {
@@ -1298,6 +1305,7 @@ struct _request_t {
 #if DELAY_POOLS
     delay_id delay_id;
 #endif
+    char *peer_login;		/* Configured peer login:password */
 };
 
 struct _cachemgr_passwd {
@@ -1369,8 +1377,8 @@ struct _StatHist {
     double min;
     double max;
     double scale;
-    hbase_f val_in;		/* e.g., log() for log-based histogram */
-    hbase_f val_out;		/* e.g., exp() for log based histogram */
+    hbase_f *val_in;		/* e.g., log() for log-based histogram */
+    hbase_f *val_out;		/* e.g., exp() for log based histogram */
 };
 
 /*
@@ -1435,7 +1443,9 @@ struct _StatCounters {
 	kb_t memory;
 	int msgs_sent;
 	int msgs_recv;
+#if USE_CACHE_DIGESTS
 	cd_guess_stats guess;
+#endif
 	StatHist on_xition_count;
     } cd;
     struct {
@@ -1605,3 +1615,58 @@ struct _htcpReplyData {
 };
 
 #endif
+
+
+struct _helper_request {
+    char *buf;
+    HLPCB *callback;
+    void *data;
+};
+
+struct _helper {
+    wordlist *cmdline;
+    dlink_list servers;
+    dlink_list queue;
+    const char *id_name;
+    int n_to_start;
+    int n_running;
+    int ipc_type;
+    time_t last_queue_warn;
+    struct {
+	int requests;
+	int replies;
+	int queue_size;
+	int avg_svc_time;
+    } stats;
+};
+
+struct _helper_server {
+    int index;
+    int rfd;
+    int wfd;
+    char *buf;
+    size_t buf_sz;
+    off_t offset;
+    struct timeval dispatch_time;
+    struct timeval answer_time;
+    dlink_node link;
+    helper *parent;
+    helper_request *request;
+    struct _helper_flags {
+	unsigned int alive:1;
+	unsigned int busy:1;
+	unsigned int closing:1;
+	unsigned int shutdown:1;
+    } flags;
+    struct {
+	int uses;
+    } stats;
+};
+
+/*
+ * use this when you need to pass callback data to a blocking
+ * operation, but you don't want to add that pointer to cbdata
+ */
+struct _generic_cbdata {
+    void *data;
+};
